'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.run = run;
exports.caseOf = caseOf;
exports.logMsg = logMsg;

var _symbolObservable = require('symbol-observable');

var _symbolObservable2 = _interopRequireDefault(_symbolObservable);

var _Observable = require('rxjs/Observable');

var _Subject = require('rxjs/Subject');

require('rxjs/add/operator/startWith');

require('rxjs/add/operator/scan');

require('rxjs/add/operator/do');

require('rxjs/add/operator/share');

require('rxjs/add/operator/map');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function isEvent(e) {
  return e && e.preventDefault && e.stopPropagation;
}

var hasRaf = typeof window !== 'undefined' && window.requestAnimationFrame;
var hasCraf = hasRaf && window.cancelAnimationFrame;

function run(program, render) {
  var opt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var _program$main = program.main,
      init = _program$main.init,
      view = _program$main.view,
      update = _program$main.update,
      subscriptions = _program$main.subscriptions;
  var onUpdate = opt.onUpdate;
  // Use requestAnimationFrame for better render performance

  var initModel = init,
      rafVal = void 0,
      sideEffect = void 0;
  var rafRender = hasRaf ? function (vdom) {
    hasCraf && rafVal && window.cancelAnimationFrame(rafVal);
    rafVal = requestAnimationFrame(function () {
      return render(vdom);
    });
  } : render;

  var msg$ = new _Subject.Subject();
  var platform = _defineProperty({
    cmd: msg$.next.bind(msg$),
    subscribe: msg$.subscribe.bind(msg$),
    complete: msg$.complete.bind(msg$)
  }, _symbolObservable2.default, function () {
    return platform;
  });

  platform.sub = platform.subscribe;

  if (typeof initModel === 'undefined') {
    throw new Error('Initial model is undefined. Pass null to explicitly set `no value`.');
  }

  if (Array.isArray(initModel) && typeof initModel[1] === 'function') {
    sideEffect = initModel[1];
    initModel = initModel[0];
  }

  var model$ = msg$.startWith(initModel).scan(function (model, msg) {
    var newModel = update(msg, model);

    if (typeof newModel === 'undefined') {
      throw new Error('Unhandled msg ' + msg.constructor.name + ' ' + JSON.stringify(msg));
    }

    if (Array.isArray(newModel) && typeof newModel[1] === 'function') {
      sideEffect = newModel[1];
      newModel = newModel[0];
    }

    onUpdate && onUpdate(msg, model, newModel);

    return newModel;
  })._do(function () {
    sideEffect && sideEffect(platform);
    sideEffect = null;
  }).share();

  var view$ = view(dispatch, model$);
  if (typeof view$ === 'function') {
    view$ = model$.map(view$);
  }

  view$.subscribe(rafRender);

  subscriptions && subscriptions(platform, initModel);

  function dispatch(msg) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    if (typeof msg === 'function') {
      return function eventHandler(e) {
        if (isEvent(e)) {
          var type = e.type,
              target = e.target;

          if (type === 'change' && target.tagName === 'INPUT' && (target.type === 'radio' || target.type === 'checkbox')) {
            msg$.next(new (Function.prototype.bind.apply(msg, [null].concat(args, [target.checked])))());
          } else {
            msg$.next(new (Function.prototype.bind.apply(msg, [null].concat(args, [target.value])))());
          }
        } else {
          msg$.next(new (Function.prototype.bind.apply(msg, [null].concat(args)))());
        }
      };
    }

    msg$.next(msg);
  }

  return {
    platform: platform,
    model$: model$
  };
}

function caseOf(msg) {
  for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i += 2) {
    var Msg = arguments.length <= i + 1 ? undefined : arguments[i + 1];
    var update = arguments.length <= i + 1 + 1 ? undefined : arguments[i + 1 + 1];
    if (msg.constructor === Msg) {
      return update.apply(undefined, _toConsumableArray(msg.args));
    }
  }
}

// https://davidwalsh.name/javascript-arguments
function getArgsNames(func) {
  // First match everything inside the function argument parens.
  var args = func.toString().match(/function\s.*?\(([^)]*)\)/)[1];

  // Split the arguments string into an array comma delimited.
  return args.split(',').map(function (arg) {
    // Ensure no inline comments are parsed and trim the whitespace.
    return arg.replace(/\/\*.*\*\//, '').trim();
  }).filter(function (arg) {
    // Ensure no undefined values are added.
    return arg;
  });
}

function logMsg(msg, log) {
  var payload = {};
  getArgsNames(msg.constructor).forEach(function (n, i) {
    return payload[n] = msg.args[i];
  });
  if (!log) console.log(msg.constructor.name, payload);else if (typeof log === 'function') log(msg.constructor.name, payload);else if (typeof log === 'string') console[log](msg.constructor.name, payload);
}